@@ -6,6 +6,36 @@
 using namespace Rcpp;
 using namespace arma;
 
+// [[Rcpp::export]]
+arma::mat normalize_byCol_dense_rcpp(arma::mat x) {
+  arma::vec sums = arma::sum(x, 0).t();
+  for (arma::uword i = 0; i < x.n_cols; ++i) {
+    if (sums[i] == 0) {
+      x.col(i).fill(0);
+    } else {
+      x.col(i) /= sums[i];
+    }
+  }
+  return x;
+}
+
+// Different from the one above, this function supports L-n normalization
+// while the one above only does sum normalization, not even necessarily L-1
+// [[Rcpp::export()]]
+arma::mat colNormalize_dense_cpp(arma::mat& x, const arma::uword L) {
+  arma::mat result(x);
+  for (int j = 0; j < x.n_cols; ++j) {
+    double norm = arma::norm(x.col(j), L);
+    if (norm == 0) {
+      continue;
+    }
+    for (int i = 0; i < x.n_rows; ++i) {
+      result(i, j) /= norm;
+    }
+  }
+  return result;
+}
+
 // ========================= Used for scaleNotCenter ===========================
 
 // [[Rcpp::export]]
@@ -27,6 +57,59 @@ arma::sp_mat scaleNotCenter_byRow_rcpp(arma::sp_mat x) {
   return x;
 }
 
+// [[Rcpp::export]]
+arma::mat safe_scale(arma::mat x, bool center, bool scale) {
+  arma::vec means = arma::mean(x, 0).t();
+  if (center) {
+    for (arma::uword i = 0; i < x.n_cols; ++i) {
+      x.col(i) -= means[i];
+    }
+  }
+
+  if (scale) {
+    arma::vec sds = arma::zeros<arma::vec>(x.n_cols);
+    for (arma::uword i = 0; i < x.n_cols; ++i) {
+      for (arma::uword j = 0; j < x.n_rows; ++j) {
+        sds[i] += x(j, i) * x(j, i);
+      }
+      sds[i] = std::sqrt(sds[i] / (x.n_rows - 1));
+
+      if (sds[i] == 0) {
+        x.col(i).fill(0);
+      } else {
+        x.col(i) /= sds[i];
+      }
+    }
+  }
+  return x;
+}
+
+
+// [[Rcpp::export]]
+arma::mat scaleNotCenter_byCol_dense_rcpp(
+  arma::mat x
+) {
+  arma::uword nrow = x.n_rows, ncol = x.n_cols;
+  arma::vec sum_of_squares(ncol);
+  for (arma::uword i = 0; i < nrow; ++i) {
+    for (arma::uword j = 0; j < ncol; ++j) {
+      sum_of_squares[j] += x(i, j) * x(i, j);
+    }
+  }
+  sum_of_squares /= (nrow - 1);
+  sum_of_squares = arma::sqrt(sum_of_squares);
+  for (arma::uword i = 0; i < nrow; ++i) {
+    for (arma::uword j = 0; j < ncol; ++j) {
+      if (sum_of_squares[j] == 0) {
+        x(i, j) = 0;
+      } else {
+        x(i, j) /= sum_of_squares[j];
+      }
+    }
+  }
+  return x;
+}
+
 // Do scaleNotCenter on each dataset annotated by `ann` (`n` levels split by
 // column), without literally splitting them and merge back.
 // x - matrix containing multiple dataset
@@ -198,9 +281,67 @@ arma::sp_mat colAggregateSums_sparse(const arma::sp_mat& x,
 //     return out;
 // }
 
+// x: n features by n selected factors
+// group: n-selected-factor integer vector, pre-transformed to 0-base,
+// from upstream kmeans clustering.
+// [[Rcpp::export()]]
+arma::mat colAggregateMedian_dense_cpp(const arma::mat& x, const arma::uvec& group, const arma::uword n) {
+  arma::mat result(x.n_rows, n);
+  for (int i = 0; i < n; ++i) {
+    arma::uvec idx = arma::find(group == i);
+    arma::mat sub_x = x.cols(idx);
+    arma::vec median = arma::median(sub_x, 1);
+    result.col(i) = median;
+  }
+  return result;
+}
+
 // [[Rcpp::export()]]
 Rcpp::NumericVector sample_cpp(const int x, const int size) {
     arma::uvec rand = arma::randperm<arma::uvec>(x);
     arma::uvec head = rand.head(size) + 1; // 0-base to R's 1-base
     return Rcpp::NumericVector(head.begin(), head.end());
 }
+
+
+// psdBulk - the pseudo-bulk matrix to be updated
+// sparseRaw - the raw expression data to be collapsed
+// featureIdx - integer vector with NAs, length equals to nrow(sparseRaw),
+//   indicating which row of `psdBulk` the current `it.row()` should be added
+//   to. Zero-based before invoking
+// repIdx - Integer vector with NAs, length equals to nrow(sparseRaw),
+//   indicating which col of `psdBulk` the current `it.col()` should be added
+//   to/ Zero-based before invoking
+// [[Rcpp::export()]]
+void updatePseudoBulkRcpp(
+  Rcpp::NumericMatrix& psdBulk,
+  const arma::sp_mat& sparseRaw,
+  const Rcpp::IntegerVector& featureIdx,
+  const Rcpp::IntegerVector& repIdx
+) {
+  for (arma::sp_mat::const_iterator it = sparseRaw.begin(); it != sparseRaw.end(); ++it) {
+    if (featureIdx[it.row()] != NA_INTEGER && repIdx[it.col()] != NA_INTEGER) {
+      psdBulk(featureIdx[it.row()], repIdx[it.col()]) += *it;
+    }
+  }
+}
+
+// out - dense matrix of n feature rows and 2 cols
+// sparseRaw - the raw expression data to be counted
+// featureIdx - integer vector with NAs, length equals to nrow(sparseRaw),
+//   indicating which row of `out` the current `it.row()` should be incremented.
+// groupVar - integer vector with NAs, length equals to ncol(sparseRaw),
+//   indicating which col of `out` the current `it.col()` should be incremented.
+// [[Rcpp::export()]]
+void updateNCellExprRcpp(
+  Rcpp::NumericMatrix& out,
+  const arma::sp_mat& sparseRaw,
+  const Rcpp::IntegerVector& featureIdx,
+  const Rcpp::IntegerVector& groupVar
+) {
+  for (arma::sp_mat::const_iterator it = sparseRaw.begin(); it != sparseRaw.end(); ++it) {
+    if (featureIdx[it.row()] != NA_INTEGER && groupVar[it.col()] != NA_INTEGER) {
+      out(featureIdx[it.row()], groupVar[it.col()])++;
+    }
+  }
+}
