@@ -87,6 +87,18 @@ test_that("scatter plots", {
     expect_gg(
         plotDimRed(pbmcPlot, colorBy = "leiden_cluster", raster = TRUE)
     )
+
+    expect_gg(plotGroupClusterDimRed(pbmcPlot))
+    do.call(expect_gg, plotGroupClusterDimRed(pbmcPlot, combinePlot = FALSE))
+
+    do.call(expect_gg, plotBarcodeRank(pbmc))
+    # Fake operation to create ATAC datasets
+    pbmcPlot@datasets$ctrl <- as.ligerDataset(dataset(pbmcPlot, "ctrl"), "atac")
+    pbmcPlot@datasets$stim <- as.ligerDataset(dataset(pbmcPlot, "stim"), "atac")
+    normPeak(pbmcPlot, "ctrl") <- normData(pbmcPlot, "ctrl")
+    normPeak(pbmcPlot, "stim") <- normData(pbmcPlot, "stim")
+    expect_gg(plotPeakDimRed(pbmcPlot, "ISG15"))
+
 })
 
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -104,6 +116,10 @@ test_that("Violin plots", {
         plotTotalCountViolin(pbmc, dot = TRUE)
     )
 
+    expect_gg(
+        plotClusterGeneViolin(pbmcPlot, "S100A8", box = TRUE, colorBy = "dataset"),
+        plotClusterGeneViolin(pbmcPlot, "S100A8", groupBy = FALSE)
+    )
     # General
     skip_if_not_installed("scattermore")
     expect_gg(
@@ -160,14 +176,17 @@ context("Density plot")
 test_that("Density plot", {
     expect_gg(
         expect_no_warning(plotDensityDimRed(pbmcPlot, splitBy = "dataset",
-                                            title = "one")),
-        expect_message(plotDensityDimRed(pbmcPlot, title = letters[1:3],
-                                         dotRaster = TRUE),
-                       "`title` has length greater than")
+                                            title = "one"))
     )
     expect_is(plotDensityDimRed(pbmcPlot, "UMAP", splitBy = "dataset",
                                 title = names(pbmcPlot), combinePlot = FALSE),
               "list")
+    skip_if_not(requireNamespace("scattermore", quietly = TRUE))
+    expect_gg(
+        expect_message(plotDensityDimRed(pbmcPlot, title = letters[1:3],
+                                         dotRaster = TRUE),
+                       "`title` has length greater than")
+    )
 })
 
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -188,6 +207,19 @@ test_that("Proportion plots", {
                  "`class1` and `class2` must be")
     expect_error(plotProportionBar(pbmcPlot, letters),
                  "`class1` and `class2` must be")
+
+    defaultCluster(pbmcPlot) <- NULL
+    expect_error(plotProportionBox(pbmcPlot), "No cluster specified nor default set")
+    defaultCluster(pbmcPlot) <- "leiden_cluster"
+    expect_error(plotProportionBox(pbmcPlot, conditionBy = "leiden_cluster"),
+                 "Condition variable must be a high level variable of the datasets")
+    expect_gg(
+        plotProportionBox(pbmcPlot, dot = TRUE),
+        plotProportionBox(pbmcPlot, conditionBy = "dataset")
+    )
+    do.call(expect_gg, plotProportionBox(pbmcPlot, splitByCluster = TRUE, dot = TRUE))
+    do.call(expect_gg, plotProportionBox(pbmcPlot, splitByCluster = TRUE, conditionBy = "dataset"))
+
 })
 
 
